/* gui.c */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <stdio.h>
#include "globals.h"
#include "./nyanko-sub.icon"

void load_font () {
    char *fontname = "9x15";
    XGCValues values;

    if ((font_info = XLoadQueryFont (display, fontname)) == NULL) {
	(void) fprintf (stderr, "%s: Cannot open 9x15 font\n", progname);
	exit (-1);
    }

    gc = XCreateGC (display, win, 0, &values);
    XSetFont (display, gc, font_info->fid);
    XSetForeground (display, gc, fg_color);
    XSetBackground (display, gc, bg_color);
    clear_gc = XCreateGC (display, win, 0, &values);
    XSetForeground (display, clear_gc, bg_color);
    XSetFillStyle (display, clear_gc, FillSolid);
}

void draw_spot (Image *image, int X, int Y) {
    int x, y;
    for (y=-radius ; y<radius+1 ; y++)
	for (x=-radius ; x<radius+1 ; x++) {
	    if ((Y+y < 0) || (X+x < 0) || (Y+y >= image->height) || (X+x >= image->width))
		continue;
	    if (XGetPixel (image, X+x, Y+y) == bg_color)
 		XPutPixel (image, X+x, Y+y, br_color);
	}
}

int prepare_subtitle (char *string, Image **put, Image **clear) {
    Pixmap pixmap;
    int x, y, string_length;
    unsigned int p_height, p_width, depth;

    printf ("ps: \"%s\"\n", string);

    depth = DefaultDepth (display, screen_num);
    string_length = strlen (string);
    p_width  = XTextWidth (font_info, string, string_length) + 2*radius;
    p_height = font_info->ascent + font_info->descent + 2*radius;

    pixmap = XCreatePixmap (display, win, p_width, p_height, depth);
    XFillRectangle (display, pixmap, clear_gc, 0, 0, p_width, p_height);
    *clear = XGetImage (display, pixmap, 0, 0, p_width, p_height, AllPlanes, ZPixmap);

    XDrawString (display, pixmap, gc, radius, font_info->ascent+radius, string, string_length);
    *put = XGetImage (display, pixmap, 0, 0, p_width, p_height, AllPlanes, ZPixmap);
    XFreePixmap (display, pixmap);

    for (y=0 ; y<(*put)->height ; y++)
	for (x=0 ; x<(*put)->width ; x++)
	    if (XGetPixel (*put, x, y) == fg_color)
		draw_spot (*put, x, y);
    return 1;
}

void put_subtitle (int x, int y, Image *image) {
    XPutImage (display, win, gc, image, 0, 0, x, y, image->width, image->height);
}

void handle_events () {
    XEvent report;

    if (XCheckMaskEvent (display, ~0, &report))    /* allow all events */
	switch (report.type) {
	  case Expose:
	    /* unless this is the last contiguous expose, don't draw the window */
	    if (report.xexpose.count != 0)
		break;
	    break;
	  case ButtonPress:
	    /* trickle, ButtonPress behaves the same as KeyPress */
	  case KeyPress:
	    XUnloadFont (display, font_info->fid);
	    XFreeGC (display, gc);
	    XCloseDisplay (display);
	    exit (1);
	  default:
	    break;
	}
}

void init (int argc, char **argv) {
    int x, y;					/* window position */
    unsigned int border_width = 4;		/* (pixels) */
    unsigned int display_width, display_height;
    char *window_name = "nyanko-sub";
    char *icon_name = "nyanko-sub";
    Pixmap icon_pixmap;
    XSizeHints *size_hints;			/* suggested window size/placement */
    int count;					/* number of allowable icon sizes */
    XIconSize *size_list;			/* icon sizes */
    XWMHints *wm_hints;				/* window manager preferences */
    XClassHint *class_hints;			/* top level window info */
    XTextProperty windowName, iconName;
    char *display_name = NULL;			/* display name ie :0.0 */

    Colormap colormap;
    XColor exact_def;
    char *foreground_color = "yellow";
    char *background_color = "white";
    char *border_color     = "black";

    /* query server for preferred settings */
    if (! (size_hints = XAllocSizeHints ())) {
	fprintf (stderr, "%s: failure allocating memory\n", progname);
	exit (0);
    }
    if (! (wm_hints = XAllocWMHints ())) {
	fprintf (stderr, "%s: failure allocating memory\n", progname);
	exit (0);
    }
    if (! (class_hints = XAllocClassHint ())) {
	fprintf (stderr, "%s: failure allocating memory\n", progname);
	exit (0);
    }

    progname = argv [0];

    /* connect to X server */
    if ( (display = XOpenDisplay (display_name)) == NULL ) {
	(void) fprintf (stderr, "%s: cannot connect to X server %s\n", progname, 
			XDisplayName (display_name));
	exit (-1);
    }

    /* get screen size from display structure macro */
    screen_num = DefaultScreen (display);
    display_width = DisplayWidth (display, screen_num);    /* root window dimensions */
    display_height = DisplayHeight (display, screen_num);

    /* default to 0,0, but should check command line */
    x = y = 0;

    /* size the window */
    win_width  = 800;
    win_height = 600;

    colormap = DefaultColormap (display, screen_num);
    if (! XParseColor (display, colormap, foreground_color, &exact_def))
	fprintf (stderr, "%s:  Color \"%s\" not in database.\n", progname, foreground_color); 
    if (! XAllocColor (display, colormap, &exact_def))
	fprintf (stderr, "%s:  Unable to allocate color.\n", progname);
    fg_color = exact_def.pixel;
    printf ("%s\t%ld\n", foreground_color, fg_color);

    if (! XParseColor (display, colormap, background_color, &exact_def))
	fprintf (stderr, "%s:  Color \"%s\" not in database.\n", progname, background_color); 
    if (! XAllocColor (display, colormap, &exact_def))
	fprintf (stderr, "%s:  Unable to allocate color.\n", progname);
    bg_color = exact_def.pixel;
    printf ("%s\t%ld\n", background_color, bg_color);

    if (! XParseColor (display, colormap, border_color, &exact_def))
	fprintf (stderr, "%s:  Color \"%s\" not in database.\n", progname, border_color); 
    if (! XAllocColor (display, colormap, &exact_def))
	fprintf (stderr, "%s:  Unable to allocate color.\n", progname);
    br_color = exact_def.pixel;
    printf ("%s\t%ld\n", border_color, br_color);

    /* create opaque window */
    win = XCreateSimpleWindow (display, RootWindow (display, screen_num), 
			       x, y, win_width, win_height, border_width, 
			       fg_color, bg_color);

    if (XGetIconSizes (display, RootWindow (display, screen_num), &size_list, &count) == 0)
	/* Window manager doesn't care about icon sizes. */
	;
    else
	/* Window manager puts <count> allowable icon sizes in <size_list>. */
	/* We should pick one of the allowable ones. */
	;

    /* Create pixmap of depth 1 (bitmap) for icon */
    icon_pixmap = XCreateBitmapFromData (display, win, icon_bits, icon_width, icon_height);

    /*
     * Set size hints for window manager.  The window manager may override these
     * settings.  Note that in a real application if size or position were set by
     * the user the flags would be UPosition and USize, and these would override
     * the window manager's preferences for this window.
     *
     * x, y, width, and height hints are now taken from the actual settings of
     * the window when mapped. Note that PPosition and PSize must be specified anyway.
     */

    /* says program (P) wants to set size, position */
    size_hints->flags = PPosition | PSize | PMinSize;
    size_hints->min_width = 800;
    size_hints->min_height = 600;

    if (XStringListToTextProperty (&window_name, 1, &windowName) == 0) {
	(void) fprintf (stderr, "%s: structure allocation for windowName failed.\n", progname);
	exit (-1);
    }
    if (XStringListToTextProperty (&icon_name, 1, &iconName) == 0) {
	(void) fprintf (stderr, "%s: structure allocation for iconName failed.\n", progname);
	exit (-1);
    }

    wm_hints->initial_state = NormalState;    /* not iconified */
    wm_hints->input = True;                   /* use wm for input */
    wm_hints->icon_pixmap = icon_pixmap;
    wm_hints->flags = StateHint | IconPixmapHint | InputHint;
 
    class_hints->res_name = progname;
    class_hints->res_class = "nyanko-sub";    /* formal name of application */

    XSetWMProperties (display, win, &windowName, &iconName, 
		      argv, argc, size_hints, wm_hints, class_hints);

    /* Select event types wanted */
    XSelectInput (display, win, 
		  ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask);

    load_font ();

    /* Display window */
    XMapWindow (display, win);
}
